import bpy
import os

current_file_path = os.path.dirname(__file__)
bl_light_icons_assets_directory = os.path.join(current_file_path, '..', 'assets')




class BL_OT_Import_Light(bpy.types.Operator):
    bl_idname = "bl_ot.import_light"
    bl_label = "Import Light"
    bl_description = "Import light presets"
    bl_options = {'REGISTER', 'UNDO'}

    def execute(self, context):
        assets_name = context.scene.LightList.options
        assets_path = bl_light_icons_assets_directory
        
        assets_blend = os.path.join(assets_path, 'assets.blend')
        assets_location = os.path.join('Object', assets_name)  

        
        with bpy.data.libraries.load(assets_blend, link=False) as (data_from, data_to):
            if assets_name in data_from.objects:
                data_to.objects = [assets_name]
            else:
                self.report({'WARNING'}, f"Object '{assets_name}' not found in '{assets_blend}'")
                return {'CANCELLED'}

        
        for obj in data_to.objects:
            for o in bpy.context.selected_objects:
                o.select_set(False)
            bpy.context.collection.objects.link(obj)
            bpy.context.view_layer.objects.active = obj
            obj.location = bpy.context.scene.cursor.location
            obj.location[2] += 10
            #select the object
            obj.select_set(True)


        
        bpy.ops.transform.translate('INVOKE_DEFAULT')

        return {'FINISHED'}
    


bl_all_presets_icons_assets_directory = os.path.join(current_file_path, '..', 'assets')

class BL_OT_Import_AllPresetsLight(bpy.types.Operator):
    bl_idname = "bl_ot.import_all_presets_light"
    bl_label = "Import Light"
    bl_description = "Import light presets"
    bl_options = {'REGISTER', 'UNDO'}

    def execute(self, context):
        assets_name = context.scene.AllPresets.options
        assets_path = bl_all_presets_icons_assets_directory
        
        assets_blend = os.path.join(assets_path, 'all_presets_assets.blend')
        assets_location = os.path.join('Object', assets_name)  

        
        with bpy.data.libraries.load(assets_blend, link=False) as (data_from, data_to):
            if assets_name in data_from.objects:
                data_to.objects = [assets_name]
            else:
                self.report({'WARNING'}, f"Object '{assets_name}' not found in '{assets_blend}'")
                return {'CANCELLED'}

        
        for obj in data_to.objects:
            for o in bpy.context.selected_objects:
                o.select_set(False)
            bpy.context.collection.objects.link(obj)
            bpy.context.view_layer.objects.active = obj
            obj.location = bpy.context.scene.cursor.location
            obj.location[2] += 10
            #select the object
            obj.select_set(True)


        
        bpy.ops.transform.translate('INVOKE_DEFAULT')

        return {'FINISHED'}