import bpy
import os
import rna_keymap_ui
from ..function.custom_icon import bl_ui_icons
from bpy.types import Menu, Operator
from bpy.types import AddonPreferences, Operator, Panel
from bpy.props import StringProperty


class PT_Better_Lighting_Npanel_ui(bpy.types.Panel):
    bl_label = "Better Lighting v2"
    bl_idname = "BL_PT_Better_Lighting_Npanel_ui"
    bl_space_type = 'VIEW_3D'
    bl_region_type = 'UI'
    bl_category = "Better Lighting"

    #header
    def draw_header(self, context):
        layout = self.layout

        layout.label(text="", icon_value=bl_ui_icons['blx64'].icon_id)

    def draw(self, context):
        layout = self.layout
        better_lighting_npanel(context, layout)



name_endwith = "##"


ui_toggle = False
ui_toggle_2 = True
ui_toggle_setting = False
ui_toggle_setting_keymap = False
ui_toggle_setting_piemenu_ui = False
ui_toggle_info = True



class light_tool_ui_toggle_panel(Operator):
    bl_idname = "ui.light_tool_ui_toggle_panel"
    bl_label = "light_tool_ui_toggle_panel"
    name: StringProperty()  

    def execute(self, context):
        global_name = self.name
        globals_dict = globals()
        if global_name in globals_dict:
            current_value = globals_dict[global_name]
            new_value = not current_value

            globals_dict[global_name] = new_value
        else:

            print(f"Global variable {global_name} not found.")

        return {'FINISHED'}



class BL_UI_LibType(bpy.types.PropertyGroup):
    type: bpy.props.EnumProperty(
        name="Library Selector",
        description="Choose an option",
        items=[ ('A', "All Presets", ""),
                ('B', "Category", ""),],
        default='A'
    ) # type: ignore




def better_lighting_npanel(context, layout):
    lib_type = context.scene.lb_ui_libtype


    obj = context.active_object

    
    box = layout.box()
    box.prop(context.scene.lb_ui_libtype, "type", text="",icon="COLLAPSEMENU")

    if lib_type.type == 'A':
        box.prop(context.scene.AllPresets, "options",text="Light")
        box.template_icon_view(context.scene.AllPresets, "options", show_labels=False, scale=8, scale_popup=8)
        box.operator("bl_ot.import_all_presets_light", text="Import", icon="IMPORT")
    elif lib_type.type == 'B':
        box.prop(context.scene.LightList, "options",text="Light")
        box.template_icon_view(context.scene.LightList, "options", show_labels=False, scale=8, scale_popup=8)
        box.operator("bl_ot.import_light", text="Import", icon="IMPORT")
    



    box = layout.box()
    op = box.operator("ui.light_tool_ui_toggle_panel", text="Light node control panel", icon=("TRIA_DOWN" if ui_toggle_2 else "TRIA_RIGHT"), emboss=False, depress=ui_toggle)
    op.name = "ui_toggle_2"
    
    if ui_toggle_2:

        if obj and obj.type == 'LIGHT':
            light = obj.data
            layout.use_property_split = False
            layout.use_property_decorate = False

            if obj and obj.type == 'LIGHT':
                if context.scene.render.engine == 'CYCLES':
                    if obj.data.node_tree:
                        box.prop(context.object.LightPresetsList, "options", text="Light Presets")
                        box.template_icon_view(context.object.LightPresetsList, "options", show_labels=False, scale=8, scale_popup=8)

                        nodes = obj.data.node_tree.nodes
                        for node in nodes:
                            if node.name.endswith(name_endwith):
                                if node.label:
                                    node_label = node.label
                                else:

                                    node_label = node.name[:-len(name_endwith)]

                                box.label(text=f"{node_label}", icon='NODETREE')

                                if hasattr(node, "draw_buttons_ext"):
                                    node.draw_buttons_ext(context, box)
                                elif hasattr(node, "draw_buttons"):
                                    node.draw_buttons(context, box)

                                value_inputs = [socket for socket in node.inputs if not socket.hide_value and not socket.is_linked]
                                if value_inputs:
                                    for socket in value_inputs:
                                        row = box.row()
                                        row.scale_x = 1
                                        row.scale_y = 1
                                        socket.draw(context, row, node, socket.name)
                    else:
                        box.label(text="Selected light has no node tree.")
                else:
                    box.label(text="Not in Cycles render engine.")
        else:
            box.label(text="No light selected.",icon="ERROR")
    else:
        pass


    
    box = layout.box()    
    op = box.operator("ui.light_tool_ui_toggle_panel", text="Setting", icon=("SETTINGS"), emboss=False, depress=ui_toggle)
    op.name = "ui_toggle_setting"
    
    if ui_toggle_setting:
        
        box2 = box.box()    
        op = box2.operator("ui.light_tool_ui_toggle_panel", text="Keymap Setting",icon=("TRIA_DOWN" if ui_toggle_setting_keymap else "TRIA_RIGHT"), emboss=False, depress=ui_toggle)
        op.name = "ui_toggle_setting_keymap"
        if ui_toggle_setting_keymap:
            draw_hotkeys(box2,"3D View")

        
        # box3 = box.box()
        # op = box3.operator("ui.light_tool_ui_toggle_panel", text="Pie Menu UI Setting",icon=("TRIA_DOWN" if ui_toggle_setting_piemenu_ui else "TRIA_RIGHT"), emboss=False, depress=ui_toggle)
        # op.name = "ui_toggle_setting_piemenu_ui"
        # if ui_toggle_setting_piemenu_ui:
        #     box3.label(text="Pie Menu UI Setting is under development.", icon="CURSOR")

    else:
        pass

    
    box = layout.box()    
    op = box.operator("ui.light_tool_ui_toggle_panel", text="Documents", icon=("HELP"), emboss=False, depress=ui_toggle)
    op.name = "ui_toggle_info"
    
    if ui_toggle_info:
        box.operator("wm.url_open", text="Documents", icon="HELP").url = "https://blendermarket.com/products/202661/docs"
        box.operator("wm.url_open", text="My Website", icon="WORLD").url = "https://blender4.com/"
    else:
        pass
    

class LIGHT_MT_pie_menu(Menu):
    bl_label = "Light Control Pie Menu"

    def draw(self, context):
        layout = self.layout


        pie = layout.menu_pie()
        cloumn = pie.box().column()
        obj = context.active_object



        if obj and obj.type == 'LIGHT':
            light = obj.data
            if obj is not None:
                cloumn.label(text=obj.name, icon="LIGHT_DATA")
            cloumn.prop(light, "type", text="Type")
            cloumn.prop(light, "color")
            cloumn.prop(light, "energy", text="Power")
            


            if light.type == 'POINT':
                cloumn.prop(light, "shadow_soft_size", text="Radius")
            elif light.type == 'SPOT':
                cloumn.prop(light, "shadow_soft_size", text="Radius")
                cloumn.prop(light, "spot_size", text="Spot Size")
                cloumn.prop(light, "spot_blend", text="Blend")
                cloumn.prop(light, "show_cone", text="Show Cone")
            elif light.type == 'SUN':
                cloumn.prop(light, "angle", text="Angle")
            elif light.type == 'AREA':
                cloumn.prop(light, "shape", text="Shape")
                cloumn.prop(light, "size", text="Size")
                
                cloumn.label(text="Beam Shape")
                cloumn.prop(light, "spread")

            #if cycles
            if context.scene.render.engine == 'CYCLES':
                cloumn.prop(light.cycles, "cast_shadow", text="Shadow")
            else:
                cloumn.prop(light,"use_shadow" ,text="Shadow")


            obj = context.active_object
            if obj and obj.type == 'LIGHT' and context.scene.render.engine == 'CYCLES':

                if obj.data.node_tree:

                    for node in obj.data.node_tree.nodes:
                        if node.name == "better_lighting":
                            column = pie.box().column()

                            column.label(text=""+context.object.LightPresetsList.options) 
                            column.template_icon_view(context.object.LightPresetsList, "options", show_labels=False, scale=8, scale_popup=8)
                            column.prop(context.object.LightPresetsList, "options", text="Light Presets") 

                            nodes = obj.data.node_tree.nodes
                            for node in nodes:
                                if node.name.endswith(name_endwith):

                                    if node.label:
                                        node_label = node.label
                                    else:
                                        
                                        node_label = node.name[:-len(name_endwith)]

                                    column.label(text=f"{node_label}", icon='NODETREE')

                                    if hasattr(node, "draw_buttons_ext"):
                                        node.draw_buttons_ext(context, column)
                                    elif hasattr(node, "draw_buttons"):
                                        node.draw_buttons(context, column)

                                    value_inputs = [socket for socket in node.inputs if not socket.hide_value and not socket.is_linked]
                                    if value_inputs:
                                        for socket in value_inputs:
                                            socket.draw(context, column, node, socket.name)

                else:

                    pass
            else:
                if obj and obj.type == 'LIGHT':

                    if obj.data.node_tree:

                        for node in obj.data.node_tree.nodes:
                            if node.name == "better_lighting":
                                column = pie.box().column()
                                column.label(text="Not in Cycles render engine or selected object is not a light.")

            
        #get render engine
        

        column = pie.box().column()
        object = obj
        

        if context.scene.render.engine == 'CYCLES':    
            column.label(text="Ray Visibility", icon="RESTRICT_VIEW_OFF")
            column.prop(object, "visible_camera", text="Camera")
            column.prop(object, "visible_diffuse", text="Diffuse")
            column.prop(object, "visible_glossy", text="Glossy")
            column.prop(object, "visible_transmission", text="Transmission")
            column.prop(object, "visible_volume_scatter", text="Volume Scatter")
        elif context.scene.render.engine == 'BLENDER_EEVEE' or context.scene.render.engine == 'BLENDER_EEVEE_NEXT':
            # column.label(text="Ray Visibility", icon="RESTRICT_VIEW_OFF")
            column.prop(light, "diffuse_factor", text="Diffuse")
            column.prop(light, "specular_factor", text="Specular")
            column.prop(light, "volume_factor", text="Volume")
            

        else:
            #render engine not supported
            column.label(text="Render engine not supported.", icon="ERROR")
            












class LIGHT_OT_control_panel_call(Operator):
    bl_idname = "light.control_panel_call"
    bl_label = "Call Light Control Pie Menu"

    @classmethod
    def poll(cls, context):
        return context.active_object and context.active_object.type == 'LIGHT'

    def execute(self, context):
        bpy.ops.wm.call_menu_pie(name="LIGHT_MT_pie_menu")
        return {'FINISHED'}







def _indented_layout(layout, level):
    indentpx = 16
    if level == 0:
        level = 0.0001   
    indent = level * indentpx / bpy.context.region.width

    split = layout.split(factor=indent)
    col = split.column()
    col = split.column()
    return col

def draw_kmi(display_keymaps, kc, km, kmi, layout, level):
    map_type = kmi.map_type

    col = _indented_layout(layout, level)

    if kmi.show_expanded:
        col = col.column(align=True)
        box = col.box()
    else:
        box = col.column()

    split = box.split()

    
    row = split.row(align=True)
    row.prop(kmi, "show_expanded", text="", emboss=False)
    row.prop(kmi, "active", text="", emboss=False)

    if km.is_modal:
        row.separator()
        row.prop(kmi, "propvalue", text="")
    else:
        row.label(text=kmi.name)
    row = layout.row()
    
    
    if map_type == 'KEYBOARD':
        row.prop(kmi, "type", text="", full_event=True)
    elif map_type == 'MOUSE':
        row.prop(kmi, "type", text="", full_event=True)
    elif map_type == 'NDOF':
        row.prop(kmi, "type", text="", full_event=True)
    elif map_type == 'TWEAK':
        subrow = row.row()
        subrow.prop(kmi, "type", text="")
        subrow.prop(kmi, "value", text="")
    elif map_type == 'TIMER':
        row.prop(kmi, "type", text="")
    else:
        row.label()

    if (not kmi.is_user_defined) and kmi.is_user_modified:
        row.operator("preferences.keyitem_restore", text="", icon='BACK').item_id = kmi.id
    else:

        pass
    
    if kmi.show_expanded:
        box = col.box()

        split = box.split(factor=0.4)
        sub = split.row()

        if km.is_modal:
            sub.prop(kmi, "propvalue", text="")
        else:

            sub.prop(kmi, "idname", text="")

        if map_type not in {'TEXTINPUT', 'TIMER'}:
            sub = split.column()
            subrow = sub.row(align=True)

            if map_type == 'KEYBOARD':
                subrow.prop(kmi, "type", text="", event=True)
                subrow.prop(kmi, "value", text="")
                subrow_repeat = subrow.row(align=True)
                subrow_repeat.active = kmi.value in {'ANY', 'PRESS'}
                subrow_repeat.prop(kmi, "repeat", text="Repeat")
            elif map_type in {'MOUSE', 'NDOF'}:
                subrow.prop(kmi, "type", text="")
                subrow.prop(kmi, "value", text="")

            if map_type in {'KEYBOARD', 'MOUSE'} and kmi.value == 'CLICK_DRAG':
                subrow = sub.row()
                subrow.prop(kmi, "direction")

            subrow = sub.row()
            subrow.scale_x = 0.75
            subrow.prop(kmi, "any", toggle=True)
            
            subrow.prop(kmi, "shift_ui", toggle=True)
            subrow.prop(kmi, "ctrl_ui", toggle=True)
            subrow.prop(kmi, "alt_ui", toggle=True)
            subrow.prop(kmi, "oskey_ui", text="Cmd", toggle=True)

            subrow.prop(kmi, "key_modifier", text="", event=True)

        
        box.template_keymap_item_properties(kmi)

        
        if not km.is_modal:
            kmm = kc.keymaps.find_modal(kmi.idname)
            if kmm:
                draw_km(display_keymaps, kc, kmm, None, layout, level + 1)
                layout.context_pointer_set("keymap", km)


kmaps_object_mode = ["light.control_panel_call"]
def draw_hotkeys(col,km_name):
    col.label(text="Keymap:", icon='TOOL_SETTINGS')
    kc = bpy.context.window_manager.keyconfigs.user
    for kmi in kmaps_object_mode:
        km2=kc.keymaps[km_name]
        kmi2=[]
        for a,b in km2.keymap_items.items():
            if a==kmi:
                kmi2.append(b)
        if kmi2:
            for a in kmi2:
                col.context_pointer_set("keymap", km2)
                draw_kmi([], kc, km2, a, col, 0)

addon_keymaps = []

def register():

    wm = bpy.context.window_manager
    km = wm.keyconfigs.addon.keymaps.new(name='3D View', space_type='VIEW_3D')
    kmi = km.keymap_items.new('light.control_panel_call', 'LEFTMOUSE', 'PRESS', ctrl=True)

    bpy.types.Scene.lb_ui_libtype = bpy.props.PointerProperty(type=BL_UI_LibType)

    addon_keymaps.append((km, kmi))


def unregister():
    
    for km, kmi in addon_keymaps:
        km.keymap_items.remove(kmi)
    addon_keymaps.clear()

    # if bpy.types.Scene.my_props:
    #     del bpy.types.Scene.my_props

if __name__ == "__main__":
    register()





